#include "TurnOut.h"

#include <UniversalAccessoryDecoder.h>

// le constructeur

TurnOut::TurnOut(){}

// les setup en mode DCC

void TurnOut::SetupDccDigital_Inverted (int TurnOutNb, int DccID, int activation, int intensity, int ledPin, int pinButton) {
    SetupDCC(TurnOutNb,DccID,activation,intensity,ledPin,pinButton,DIGITAL_INVERTED);
}
void TurnOut::SetupDccDigital (int TurnOutNb, int DccID, int activation, int intensity, int ledPin, int pinButton) {
    SetupDCC(TurnOutNb,DccID,activation,intensity,ledPin,pinButton,DIGITAL);
}

// les setup en mode TCO

void TurnOut::SetupDigital_Inverted (int TurnOutNb,int activation, int intensity) {
    Setup(TurnOutNb,activation,intensity,DIGITAL_INVERTED,false);
}
void TurnOut::SetupDigital (int TurnOutNb, int activation, int intensity) {
    Setup(TurnOutNb,activation,intensity,DIGITAL,true);
}

// le setup interne en mode TCO

void TurnOut::Setup(int nbTurnOut, int activateTime, int intensityTime, PORT_TYPE porttype, bool mode)
{

	int i;
	int indexPin = FIRST_PIN;
	int IdDcc = FIRST_DCC;
	UAD_StartSetup();

	button.Setup(nbTurnOut);

// un bouton par aiguillage

	for (i = 0; i < nbTurnOut; i++) {
		button.Add(new ButtonsCommanderPush(2));
		PUSH(button, i)->AddDccId(IdDcc + i, 0);
		PUSH(button, i)->AddDccId(IdDcc + i, 1);
		PUSH(button, i)->Setup(indexPin);
		indexPin = indexPin + 5;
	}

// le driver pour 2 leds + 2 bobines

	arduino = new DriverArduino(4 * nbTurnOut, 0);
	arduino->Setup();

	int port = 0;
	indexPin = FIRST_PIN + 1;
	for (i = 0; i < nbTurnOut; i++) {
		for (int j = 0; j < 4; j++) { arduino->SetupPortMotor(port + j, indexPin + j, porttype); }
		port = port + 4;
		indexPin = indexPin + 5;
	}

	port = 0;

// les accessoires 2 leds + 2 bobines

	turnOuts.Setup(4 * nbTurnOut);

	for (i = 0; i < nbTurnOut; i++) {
		turnOuts.Add(new AccessoryLight(1000 + port, 0));       // led 1
		LIGHT(turnOuts, port)->Setup(arduino, port);
		turnOuts.Add(new AccessoryLight(1000 + port, 1));       // led 2
		LIGHT(turnOuts, port + 1)->Setup(arduino, port + 1);
		turnOuts.Add(new AccessoryMotorTwoWays(IdDcc + i, 0, activateTime));
		MOTOR2WAYS(turnOuts, port + 2)->Setup(arduino, port + 2, intensityTime);    // bobine 1
		turnOuts.Add(new AccessoryMotorTwoWays(IdDcc + i, 1, activateTime));
		MOTOR2WAYS(turnOuts, port + 3)->Setup(arduino, port + 3, intensityTime);    // bobine 2
		port = port + 4;
	}

// gestion des leds

	light.Setup(2 * nbTurnOut);

	port = 0;
	for (i = 0; i < nbTurnOut; i++) {
		GroupState *droite = new GroupState(IdDcc + i, 0);  // la bobine droite
		droite->Setup(2);
		if (mode) {                                 // si DIGITAL
			droite->Add(turnOuts[port], LIGHTOFF);
			droite->Add(turnOuts[port + 1], LIGHTON);
		}
		else {                                  // si DIGITAL_INVERTED
			droite->Add(turnOuts[port], LIGHTON);
			droite->Add(turnOuts[port + 1], LIGHTOFF);
		}
		port = port + 4;
		light.Add(droite);
	}

	port = 0;

	for (i = 0; i < nbTurnOut; i++) {
		GroupState *gauche = new GroupState(IdDcc + i, 1);  // la bobine gauche
		gauche->Setup(2);
		if (mode) {                                // si DIGITAL
			gauche->Add(turnOuts[port], LIGHTON);
			gauche->Add(turnOuts[port + 1], LIGHTOFF);
		}
		else {                                   // si DIGITAL_INVERTED
			gauche->Add(turnOuts[port], LIGHTOFF);
			gauche->Add(turnOuts[port + 1], LIGHTON);
		}
		port = port + 4;
		light.Add(gauche);
	}


	UAD_EndSetup();  // fin du setup interne en mode TCO
}

// le setupinterne en mode DCC

void TurnOut::SetupDCC(int nbTurnOut, int IdDcc, int activateTime, int intensityTime, int ledStatusPin, int buttonPin, PORT_TYPE porttype)
{

	int i;
	int indexPin = FIRST_PIN;

	UAD_StartSetup();
	dcc.Setup(0x00, 0x00, KDCC_INTERRUPT);
	dcc.SetStatusLedPin(ledStatusPin);

// un bouton test circulaire sur les adresses DCC

	button.Setup(1, new ButtonsCommanderPush(nbTurnOut * 2));
	for (i = 0; i < nbTurnOut; i++) {
		PUSH(button, 0)->AddDccId(IdDcc + i, 0);
		PUSH(button, 0)->AddDccId(IdDcc + i, 1);
	}
	PUSH(button, 0)->Setup(buttonPin);

// driver pour les deux bobines

	arduino = new DriverArduino(2 * nbTurnOut, 0);
	arduino->Setup();

	int port = 0;

	for (i = 0; i < nbTurnOut; i++) {
		for (int j = 0; j < 2; j++) { arduino->SetupPortMotor(port + j, indexPin + j, porttype); }
		port = port + 2;
		indexPin = indexPin + 2;
	}
	port = 0;

	// les accessoires bobines

	turnOuts.Setup(2 * nbTurnOut);

	for (i = 0; i < nbTurnOut; i++) {
		turnOuts.Add(new AccessoryMotorTwoWays(IdDcc + i, 0, activateTime));
		MOTOR2WAYS(turnOuts, port)->Setup(arduino, port, intensityTime);
		turnOuts.Add(new AccessoryMotorTwoWays(IdDcc + i, 1, activateTime));
		MOTOR2WAYS(turnOuts, port + 1)->Setup(arduino, port + 1, intensityTime);
		port = port + 2;
	}

	UAD_EndSetup();   // fin du setup interne en mode DCC
}

// la boucle en mode DCC

void TurnOut::LoopDcc() {
if (dcc.Loop()) {
    turnOuts.Loop();
    button.Loop();
    }
}

// la boucle en mode TCO

void TurnOut::Loop() {
    turnOuts.Loop();
    light.Loop();
    button.Loop();
}



