//-------------------------------------------------------------------
#ifndef __accessoryGroup_H__
#define __accessoryGroup_H__
//-------------------------------------------------------------------

#include "Accessory.hpp"

//-------------------------------------------------------------------

//-------------------------------------------------------------------

#ifndef NO_GROUP

// One item : Motor1/left or Light18 -/On
// The delay gives a duration of the movement, to ensure that this movement is finished before starting the next one. Only used in asynchronous group state.
class GroupStateItem
{
	public:
		Accessory *pAccessory;
		ACC_STATE State;
		unsigned int Delay;

		GroupStateItem(Accessory *inpAccessory, ACC_STATE inState, unsigned int inDelay = 0) { this->pAccessory = inpAccessory; this->State = inState; this->Delay = inDelay; }
};

// A complete state of the group : Light18/On and Motor3/Right .
class GroupState
{
	public:
		int DccIdDecoder;
		byte DccIdAccessory;
		GroupStateItem* *pItems;
		bool Synchrone;

	private:
		unsigned char size;
		unsigned char addCounter;
		int actionProgressCounter;
		unsigned long startActionTime;

	public:
		GroupState(int DccId, byte inDccIdAccessory, bool inSynchrone = true);

		void Setup(unsigned char inSize, GroupStateItem *inpFirstItem, ...);
		void Setup(unsigned char inSize);
		void Set(unsigned char inIndex, GroupStateItem *inpItem);
		GroupStateItem *operator[](unsigned char idx);		
	
		void StartAction();
		void ResetAction()	{ this->actionProgressCounter = -1; }
		bool IsActionPending()	{ return this->actionProgressCounter >= 0; }

		void Loop();

#ifdef DEBUG_MODE
		void CheckIndex(unsigned char inIndex, const __FlashStringHelper *infunc);
#endif
	public:
		unsigned char Add(GroupStateItem *inpItem);
		unsigned char Add(Accessory *inpAccessory, ACC_STATE inState);
};

// Describes static data only intialized once at the beginning of execution.

class AccessoryGroup;

class StaticGroupData
{
public:
	// List of pointers	to all groups declared.
	// The commanders can enumerate this list to find thing to move...
	AccessoryGroup* *pAccessoryGroupFullList;
	int AccessoryGroupSize;
	int AccessoryGroupAddCounter;

	StaticGroupData();
};

// Group of accessories defining a list of states of these acessories.
class AccessoryGroup
{
	public:
		static StaticGroupData StaticData;

	private:
		unsigned char size;
		unsigned char addCounter;

	public:
		GroupState* *pStates;
		int CurrentState;	// Must be -1 !

	public:
		AccessoryGroup();

		void Setup(int inSize);
		void Setup(int inSize, GroupState *inpFirstState, ...);
		void Set(unsigned char inIndex, GroupState *inpState);
		void AddRange(const AccessoryGroup &inGroup);
		GroupState *operator[](unsigned char idx);
		GroupState *GetByID(int inDccId, byte inDccIdAccessory);
		int IndexOf(GroupState *inpState);
		int IndexOf(int inDccId, byte inDccIdAccessory);
		static void AddAccessoryGroup(AccessoryGroup *inGroup);

		void StartAction(unsigned char inStateIndex);
		void ResetAction()	{ this->CurrentState = -1; }
		bool IsActionPending();

		bool Loop();
		void ButtonToggle(int inButton);
		bool DCCToggle(int inDccId, byte inDccIdAccessory);

#ifdef DEBUG_MODE
		void CheckIndex(unsigned char inIndex, const __FlashStringHelper *infunc);
#endif
	public:
		unsigned char Add(GroupState *inpState);
};
#endif

//-------------------------------------------------------------------

//===================================================================
// -> DO NOT WRITE ANYTHING BETWEEN HERE...
// 		This section is reserved for automated code generation
// 		This process tries to detect all user-created
// 		functions in main_sketch.cpp, and inject their  
// 		declarations into this file.
// 		If you do not want to use this automated process,  
//		simply delete the lines below, with "&MM_DECLA" text 
//===================================================================
//---- DO NOT DELETE THIS LINE -- @MM_DECLA_BEG@---------------------
//---- DO NOT DELETE THIS LINE -- @MM_DECLA_END@---------------------
// -> ...AND HERE. This space is reserved for automated code generation!
//===================================================================


//-------------------------------------------------------------------
#endif
//-------------------------------------------------------------------
