//-------------------------------------------------------------------
#ifndef __accessoryLight_H__
#define __accessoryLight_H__
//-------------------------------------------------------------------

#include "Accessory.hpp"

//-------------------------------------------------------------------

//-------------------------------------------------------------------

#ifndef NO_LIGHT

#define LIGHTON		STATE_FIRST
#define LIGHTBLINK	STATE_SECOND
#define LIGHTOFF	STATE_NONE

// This class describes a basic light.

#define LIGHT(list, nb)	((AccessoryLight *) (list)[nb])

class AccessoryLight : public Accessory
{
	private:
		Driver *pDriver;
		unsigned char port;

		bool flashingState;	// true : light on during flashing, false : light off during flashing.
		unsigned char fadingCurrentValue;
		char fadingCurrentDirection;	// 1 or -1
		unsigned char fadingStep;
		unsigned char fadingDelay;

	public:
		AccessoryLight(int inDccId, byte inDccIdAccessory, unsigned long inBlinkDuration = 0);
		
		inline bool IsOn() const { return this->state == LIGHTON; }
		inline bool IsFlashing() const { return this->state == LIGHTBLINK; }
		inline bool IsFading() const { return this->fadingStep > 0; }
		inline unsigned char GetPort() const { return this->port; }

	public:
		void SetState(ACC_STATE instate);
		ACC_STATE Toggle();
		inline ACCESSORYTYPE GetAccessoryType() const { return  ACCESSORYLIGHT; }
		void SetFading(unsigned char inStep, unsigned char inDelay);
		void Setup(Driver *inpDriver, unsigned char inPort, int inIntensity = 255);

	public:
		void Move(int inDccId, byte inDccAccessory);
		virtual void LightFading(unsigned char inValue);
		virtual void LightOn();
		virtual void LightOff();
		virtual void LightOnRaw();
		virtual void LightOffRaw();
		inline virtual void Blink() { this->state = LIGHTBLINK; }
		inline bool IsGroupActionPending() const	{ return false; }
		void StartAction();
		bool ActionEnded();
};
#endif

//-------------------------------------------------------------------

//===================================================================
// -> DO NOT WRITE ANYTHING BETWEEN HERE...
// 		This section is reserved for automated code generation
// 		This process tries to detect all user-created
// 		functions in main_sketch.cpp, and inject their  
// 		declarations into this file.
// 		If you do not want to use this automated process,  
//		simply delete the lines below, with "&MM_DECLA" text 
//===================================================================
//---- DO NOT DELETE THIS LINE -- @MM_DECLA_BEG@---------------------
//---- DO NOT DELETE THIS LINE -- @MM_DECLA_END@---------------------
// -> ...AND HERE. This space is reserved for automated code generation!
//===================================================================


//-------------------------------------------------------------------
#endif
//-------------------------------------------------------------------
