//-------------------------------------------------------------------
#ifndef __commander_H__
#define __commander_H__
//-------------------------------------------------------------------

#include "Accessories.hpp"
#include "AccessoryGroup.hpp"

//-------------------------------------------------------------------

//-------------------------------------------------------------------

#ifdef DEBUG_MODE
#endif

class Commander;

// Describes static data only intialized once at the beginning of execution.

class StaticCommanderData
{
public:
	// List of all commanders declared.
	// For instance DCCCommander overrides a function PriorityLoop which must be called as often as possible.
	// Other loops (asscessories, groups) have just to call Commander::StaticData::CommanderPriorityLoop() to be sure
	// to call priority functions very often.
	Commander* *pCommanderList;
	int CommanderSize;
	int CommanderAddCounter;

	StaticCommanderData();

	void CommanderPriorityLoop();
};

class Commander
{
	public:
		static StaticCommanderData StaticData;

	public:
		inline Commander() { AddCommander(this); }
		
	public:
		virtual void Setup() {}
		inline virtual bool Loop() { return false; }
		inline virtual void PriorityLoop() { }

		static void AddAccessories(const Accessories &inAccessories);
		static void AddAccessoryGroup(AccessoryGroup *inGroup);
		static void AddCommander(Commander *inCommander);

#ifdef DEBUG_MODE
	public:
		void CheckIndex(unsigned char inIndex, const __FlashStringHelper *infunc);
#endif
};

//-------------------------------------------------------------------

//===================================================================
// -> DO NOT WRITE ANYTHING BETWEEN HERE...
// 		This section is reserved for automated code generation
// 		This process tries to detect all user-created
// 		functions in main_sketch.cpp, and inject their  
// 		declarations into this file.
// 		If you do not want to use this automated process,  
//		simply delete the lines below, with "&MM_DECLA" text 
//===================================================================
//---- DO NOT DELETE THIS LINE -- @MM_DECLA_BEG@---------------------
//---- DO NOT DELETE THIS LINE -- @MM_DECLA_END@---------------------
// -> ...AND HERE. This space is reserved for automated code generation!
//===================================================================


//-------------------------------------------------------------------
#endif
//-------------------------------------------------------------------
