// UAD.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "math.h"

void setup();
void loop();

int _tmain(int argc, _TCHAR* argv[])
{
	setup();

	while (true)
		loop();

	return 0;
}

//-----------------------------------------
//			ino PART
//-----------------------------------------
#include "UniversalAccessoryDecoder.h"

//------------------------------------------------------------------------------
// SignalArduino declaration

class SignalArduino : public AccessoryGroup
{
protected:
	byte nbLeds;
	unsigned char *accNb;

public:
	SignalArduino(Accessories *inpAccessories, DriverArduino *inpDriver, byte inNbLeds, int *inpPins, int inFirstPort = 0);
	virtual void SetupSignal(Accessories *inpAccessories, int inStartingDcc);
};

//------------------------------------------------------------------------------
// SignalArduino definition

SignalArduino::SignalArduino(Accessories *inpAccessories, DriverArduino *inpDriver, byte inNbLeds, int *inpPins, int inFirstPort)
{
	this->nbLeds = inNbLeds;
	for (int led = 0; led < inNbLeds; led++)
	{
		inpDriver->SetupPortMotor(inFirstPort + led, inpPins[led], DIGITAL_INVERTED);
	}

	this->accNb = new unsigned char[inNbLeds];

	for (int led = 0; led < inNbLeds; led++)
	{
		this->accNb[led] = inpAccessories->Add(new AccessoryLight(1000 + inFirstPort, led));
	}

	for (int led = 0; led < inNbLeds; led++)
	{
		// Le numro de la led est aussi le numro du port.
		LIGHT(*inpAccessories, this->accNb[led])->Setup(inpDriver, inFirstPort + led);
	}
}

void SignalArduino::SetupSignal(Accessories *inpAccessories, int inStartingDcc)
{
	int led_on = 0;

	this->Setup((int)this->nbLeds + 1);

	// Usage des dcc codes :
	//                 Led  0    1    2
	// inStartingDcc   / 0  on  off  off
	// inStartingDcc   / 1  off on   off
	// inStartingDcc+1 / 0  off off  on
	// inStartingDcc+1 / 1  off off  off


	for (int etat = 0; etat < this->nbLeds; etat++)
	{
		int dccDecoder;
		byte dccState;

		switch (etat)
		{
		case 0:
			dccDecoder = inStartingDcc;
			dccState = 0;
			break;
		case 1:
			dccDecoder = inStartingDcc;
			dccState = 1;
			break;
		case 2:
			dccDecoder = inStartingDcc + 1;
			dccState = 0;
			break;
		}

		// Each state describes 2 leds off and one on.
		GroupState *pState = new GroupState(dccDecoder, dccState);
		pState->Setup(this->nbLeds);

		for (int led = 0; led < this->nbLeds; led++)
		{
			GroupStateItem *pItem = new GroupStateItem((*inpAccessories)[this->accNb[led]], led_on == led ? LIGHTON : LIGHTOFF);
			pState->Add(pItem);
		}
		this->Add(pState);
		led_on++;
	}

	// The last state is only to set off all leds.
	GroupState *pState = new GroupState(inStartingDcc + 1, 1);
	pState->Setup(this->nbLeds);

	for (int led = 0; led < this->nbLeds; led++)
	{
		GroupStateItem *pItem = new GroupStateItem((*inpAccessories)[this->accNb[led]], LIGHTOFF);
		pState->Add(pItem);
	}
	this->Add(pState);

	// Light on at startup
	for (int led = 0; led < this->nbLeds; led++)
	{
		((AccessoryLight *)(*inpAccessories)[this->accNb[led]])->LightOn();
		delay(200);
	}

	delay(200);

	for (int led = 0; led < this->nbLeds; led++)
	{
		((AccessoryLight *)(*inpAccessories)[this->accNb[led]])->LightOff();
		delay(200);
	}
	delay(200);
	this->DCCToggle(inStartingDcc, 0);
}

//------------------------------------------------------------------------------
// Classic INO area

/* kDCC_INTERRUPT values :
Board         int.0   int.1   int.2   int.3   int.4   int.5
Uno, Ethernet   2      3
Mega2560        2      3      21      20      19      18
Leonardo        3      2      0       1       7
*/
#define kDCC_INTERRUPT            3    // pin 20 for Mega2560

#define NB_LEDS		3
#define NB_ETATS	3
#define NB_FEUX		16

SignalArduino* signaux[NB_FEUX];

// Accessories

Accessories accessories;
DccCommander dccCommander;
ButtonsCommander buttonsCommander;

// Drivers

DriverArduino *arduino;
/*
int pins[NB_FEUX][NB_ETATS] = {  // {Rouge, Jaune, Vert }
{ 49, 48, 47 },
{ 46, 45, 44 },
{ 43, 42, 41 },
{ 40, 39, 38 },
{ 37, 36, 35 },
{ 34, 33, 32 },
{ 31, 30, 29 },
{ 28, 27, 26 },
{ 25, 24, 23 },
{ 19, 18, 17 },
{ 16, 15, 14 },
{ 13, 12, 11 },
{ 10, 9, 8 },
{ 7, 6, 5 },
{ 4, 3, 2 },
{ A12, A11, A10 },
{ A15, A14, A13 }
};
*/

int pins[NB_FEUX][NB_ETATS] = {  // {Rouge, Jaune, Vert }
	{ 11, 10, 9 },
	{ 8, 7, 6 },
	{ 5, 4, 25 },
	{ 27, 29, 31 },
	{ 28, 30, 32 },
	{ 34, 36, 38 },
	{ 40, 42, 44 },
	{ 46, 48, 50 },
	{ 13, 12, 3 },
	{ 2, 14, 15 },
	{ 16, 17, 18 },
	{ 19, 22, 23 },
	{ 24, 26, 33 },
	{ 35, 37, 39 },
	{ 41, 43, 45 },
	{ 47, 49, 51 }
};


int dcc_codes[NB_FEUX] = { 100, 102, 104, 106, 108, 110, 112, 114, 116, 118, 120, 122, 124, 126, 128, 130 };

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Setup
//
void setup()
{
	UAD_StartSetup();

	dccCommander.Setup(0x00, 0x00, kDCC_INTERRUPT);
	dccCommander.SetStatusLedPin(A12);

	buttonsCommander.Setup(1,
		new ButtonsCommanderPush(NB_FEUX * NB_ETATS)
		);

	// Ce petit bouton va permettre de passer en revue tous les codes dcc des feux en squence...
	for (int feu = 0; feu < NB_FEUX; feu++)
	{
		for (int etat = 0; etat < NB_ETATS; etat++)
		{
			PUSH(buttonsCommander, 0)->AddDccId(dcc_codes[feu], etat);
		}
	}

	PUSH(buttonsCommander, 0)->Setup(56);           // port A2

	// 48 ports  initialiser
	arduino = new DriverArduino(NB_LEDS * NB_FEUX, 0);
	arduino->Setup();

	accessories.Setup(NB_LEDS * NB_FEUX);

	for (int feu = 0; feu < NB_FEUX; feu++)
	{
		signaux[feu] = new SignalArduino(&accessories, arduino, NB_LEDS, pins[feu], feu * NB_LEDS);
		signaux[feu]->SetupSignal(&accessories, dcc_codes[feu]);
	}

	UAD_EndSetup();
}

void loop()
{
	if (dccCommander.Loop())
	{
		accessories.Loop();
		buttonsCommander.Loop();
		for (int feu = 0; feu < NB_FEUX; feu++)
			signaux[feu]->Loop();
	}
}

//-----------------------------------------
//			end ino PART
//-----------------------------------------
